"""Command line argument parsers."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import argparse
import logging
import pathlib

# locals
from climax.core.constants import (CORDEX_VARIABLES, CORDEX_EXPERIMENTS,
                                   EUROCORDEX_GCMS,  EUROCORDEX_RCMS,
                                   CDO_RESAMPLING_MODES)

# epilogue to display at the end of each parser
EPILOGUE = 'Author: Daniel Frisinghelli, daniel.frisinghelli@gmail.com'

# module level logger
LOGGER = logging.getLogger(__name__)


# parser to preprocess Cordex data: climax.main.preprocess.py
def preprocess_parser():

    # define command line argument parser
    parser = argparse.ArgumentParser(
        description='Reproject and resample Cordex data to a target grid.',
        epilog=EPILOGUE,
        formatter_class=lambda prog: argparse.RawDescriptionHelpFormatter(
            prog, max_help_position=50, indent_increment=2))

    # positional arguments

    # positional argument: path to the target grid file
    parser.add_argument('grid', type=pathlib.Path,
                        help='Path to the target grid file.')

    # positional argument: path to search for Cordex NetCDF files
    parser.add_argument('source', type=pathlib.Path,
                        help='Path to search for Cordex NetCDF files.')

    # positional argument: path to save the reprojected NetCDF files
    parser.add_argument('target', type=pathlib.Path,
                        help='Path to save the remapped Cordex NetCDF files.')

    # optional arguments

    # default values
    default = '(default: %(default)s)'

    # optional argument: name of the variable of interest
    parser.add_argument('-var', '--variable', type=str,
                        help='Name of the variable of interest.',
                        choices=CORDEX_VARIABLES, default=None, metavar='')

    # optional argument: name of the global climate model
    parser.add_argument('-g', '--gcm', type=str,
                        help=('Name of the global climate model {}.'
                              .format(default)),
                        choices=EUROCORDEX_GCMS, default=None, metavar='')

    # optional argument: name of the Cordex scenario
    parser.add_argument('-s', '--scenario', type=str,
                        help=('Name of the Cordex scenario {}.'
                              .format(default)),
                        choices=CORDEX_EXPERIMENTS, default=None, metavar='')

    # optional argument: ensemble identifier
    parser.add_argument('-e', '--ensemble', type=str,
                        help=('Ensemble identifier {}.'.format(default)),
                        default=None, metavar='')

    # optional argument: name of the regional climate model
    parser.add_argument('-r', '--rcm', type=str,
                        help=('Name of the regional climate model {}.'
                              .format(default)),
                        choices=EUROCORDEX_RCMS, default=None, metavar='')

    # optional argument: version of the regional climate model
    parser.add_argument('-v', '--version', type=str,
                        help=('Version of the regional climate model {}.'
                              .format(default)), default=None, metavar='')

    # optional argument: resampling mode
    parser.add_argument('-m', '--mode', type=str,
                        help='Resampling mode {}.'.format(default),
                        default='bilinear', choices=CDO_RESAMPLING_MODES,
                        metavar='')

    # optional argument: whether to overwrite files
    parser.add_argument('-o', '--overwrite', type=bool,
                        help='Overwrite existing files {}.'.format(default),
                        default=False, nargs='?', const=True, metavar='')

    # optional argument: whether to aggregate time periods of simulations
    parser.add_argument('-a', '--aggregate', type=bool,
                        help=('Aggregate time periods of simulations to a '
                              'single NetCDF file {}.'.format(default)),
                        default=False, nargs='?', const=True, metavar='')

    # optional argument: whether to remove single netcdf files
    parser.add_argument('-rm', '--remove', type=bool,
                        help=('Remove individual NetCDF files of different '
                              'time periods. Only applies when -a is True {}.'
                              .format(default)),
                        default=False, nargs='?', const=True, metavar='')

    # optional argument: dry run, print files which would be processed
    parser.add_argument('-d', '--dry-run', type=bool,
                        help=('Print files which would be processed {}.'
                              .format(default)), default=False, nargs='?',
                        const=True, metavar='')

    # optional argument: csv file describing Cordex simulations
    parser.add_argument('-f', '--file', type=pathlib.Path,
                        help=('Path to EuroCordex inventory .csv file. {}.'
                              .format(default)), default=None, metavar='')

    return parser
